==> one-page-app-0.1.0.tgz: package/index.js <==
var dirtkey = require('dirtkey'),
    conf = new(dirtkey.conf)({host:'in1.iriscouch.com',port:'80',db:'screenshot'}),
    fs = require('fs');

var locals = {
    author:'in1'
};

function initApp(req,res){
    
    // this is run once to setup new app or to reset all app data
    
    // one time copy of orig data files
    //fs.createReadStream(__dirname + '/opa.db_').pipe(fs.createWriteStream(__dirname + '/opa.db'));
    //fs.createReadStream(__dirname + '/sections.db_').pipe(fs.createWriteStream(__dirname + '/section.db'));
    //fs.createReadStream(__dirname + '/snippet.db_').pipe(fs.createWriteStream(__dirname + '/snippet.db'));
    //fs.createReadStream(__dirname + '/user.db_').pipe(fs.createWriteStream(__dirname + '/user.db'));
    
    res.redirect("/admin");
}

exports.index = function(req,res){
    console.log("show home page");

    var appDb = dirtkey('opa.db'),
        pageDb = dirtkey('page.db'),
        sectionsDb;
        
    appDb.on('load', function() {
        var app = appDb.get('app');
        
        if (typeof app === "undefined"){
            initApp(req,res);
        }
        
        sectionsDb = dirtkey('sections.db');
        sectionsDb.on('load', function() {
            if (req.session && typeof req.session.once=="undefined") {
                locals.once=pageDb.once;
                req.session.once=1;
            }
            res.render('index.ejs', {locals:locals,sections:sectionsDb,app:appDb._docs,page:pageDb._docs,msg:req.query.msg,err:req.query.err});
        });
    });
};

exports.login = function(req,res){
    console.log("show login page");

	var appDb = dirtkey('opa.db'),
        pageDb = dirtkey('page.db'),
		sectionsDb;
		
	appDb.on('load', function() {
		sectionsDb = dirtkey('sections.db');
		sectionsDb.on('load', function() {
			res.render('login', {locals:locals,sections:sectionsDb,app:appDb._docs,page:pageDb._docs,err:req.query["err"]});
		});
	});
};

exports.login.post = function(req,res){
	console.log("logging in");
		
	var userDb = dirtkey('user.db'),
		username = req.body["username"],
		password = req.body["password"];
		
	if (typeof username=="undefined" || typeof password=="undefined") {
		//res.render('error', {msg:'You need to enter a username and password.'});
		res.redirect('/login?err=You need to enter a username and password.');
	}
	else {
		userDb.on('load', function() {
			var user = userDb.get(username);
			if (typeof user=="undefined" || user.password!=password) {
				res.redirect('/login?err=User not found or wrong password.');
			}
			else if (!user.admin && user.admin!=1) {
				res.redirect('/login?err=You do not have permission to access this page.');
			}
			else {
				/* set session login here*/
				req.session.loggedIn=true;
				req.session.username=username;
				res.redirect('/admin');
			}
		});
	}
};

exports.logout = function(req,res){
	console.log("logging out");
    
	delete req.session.loggedIn;
	res.redirect('/');
};

exports.admin = function(req,res){
	console.log("getting admin page..");

    locals.date = new Date().toLocaleDateString();
		
	if (req.session.loggedIn) {
		var appDb = dirtkey('opa.db');
		appDb.on('load', function() {
            var app = appDb.get('app')||{app:{}},
                page = appDb.get('page')||{page:{}},
                sectionsDb = dirtkey('sections.db');
			
            if (typeof app === "undefined"){
               initApp(req,res);
            }
            
            sectionsDb.on('load', function() {
				
				var snippetsDb = dirtkey('snippet.db');
				snippetsDb.on('load', function() {
					res.render('admin', {locals:locals,sections:sectionsDb,snippets:snippetsDb,app:app,page:page,msg:req.query.msg,err:req.query.err});
				});
			});
		});
		
	}
	else {
		res.redirect('/login');
	}
};

exports.admin.app = function(req,res){
	console.log("saving app settings..");

	var appDb = dirtkey('opa.db');
	console.log('save app.');
	
	appDb.on('load', function() {
		console.log('save app----');
		//appDb.set('app',{title:req.body["title"],description:req.body["description"],keywords:req.body["keywords"]}, function() {
		appDb.set('app',req.body, function() {
			console.log('Added mobile %s.', appDb.get('app'));
		});
	});
	
	appDb.on('drain', function() {
		console.log('app is saved on disk.');
		
		var sectionsDb = dirtkey('sections.db');
		sectionsDb.on('load', function() {
			
			var snippetsDb = dirtkey('snippet.db');
			snippetsDb.on('load', function() {
				res.render('admin', {locals:locals,sections:sectionsDb,snippets:snippetsDb,app:appDb.get('app'),page:appDb.get('page'),msg:"App Settings Saved."});
			});
		});
	
		//res.redirect("/admin");
	});
};

exports.admin.page = function(req,res){
	console.log("saving page settings..");

	var appDb = dirtkey('opa.db');
	console.log('save page.');
	
	appDb.on('load', function() {
		appDb.set('page',req.body, function() {
			console.log('Saved page %s.', appDb.get('app'));
		});
	});
	
	appDb.on('drain', function() {
		console.log('app page is saved on disk.');
		res.redirect("/admin");
	});
};

exports.admin.sections = function(req,res){
	console.log("creating new section..");

	var sectionsDb = dirtkey('sections.db'),
		key = req.body["section"],
		vals = req.body;
		vals.cssClass = "fill"; // always add this to the CSS class
			
	if (typeof key==="undefined" || key==="") {
		res.redirect("/admin#form"+key);
	}
	else {
		key = key.replace(/[^a-zA-Z0-9]/g,''); // strip special chars.
		sectionsDb.on('load', function() {
			sectionsDb.set(key,vals, function() {
				console.log('Added content %s', sectionsDb.get(key));
				res.redirect("/admin#form"+key);
				
			});
		});
	}
};

exports.admin.sections.post = function(req,res){
	console.log("saving section..");

	var	sectionsDb = dirtkey('sections.db'),
		key = req.params["k"],
		vals = req.body;
	
	console.log("saving section:"+key);
	
	if (typeof key!="undefined") {
		sectionsDb.on('load', function() {
			sectionsDb.set(key,vals, function() {
				console.log('Added content', sectionsDb.get(key));
			});
			//res.redirect("/admin");
			res.send('Section saved.');
		});
	}
	else {
		res.send('Section not saved.');
	}
	
};

exports.admin.sections.del = function(req,res){
	console.log("deleting section..");

	var sectionsDb = dirtkey('sections.db'),
		key = req.params["k"],
		section = req.body["section"];
	
	if (typeof key!="undefined") {
		sectionsDb.rm(key, function() {
			console.log('Deleted section');
		});
		
		sectionsDb.on('drain', function() {
			res.send('Section deleted.');
		});
	}
	else {
		res.send('Section delete failed.')
	}
};

exports.admin.sections.all = function(req,res){
	console.log("saving all sections..");

	var appDb = dirtkey('opa.db'),
		sectionsDb = dirtkey('sections.db'),
		sections = req.body["section"],
		contents = req.body["content"];
	
	//console.log(sections.toString());
	//console.log(contents.toString());
	
	var i=0;
	sections.forEach(function() {
		sectionsDb.set(sections[i],{content:contents[i]}, function() {
			console.log('Added content %s.', sectionsDb.get(sections[i]));
		});
		i++;
	});
	
	res.render('admin', {locals:locals,app:appDb,sections:sectionsDb,msg:"Saved Sections"});
};

exports.admin.sections.all.post = function(req,res){
	console.log("saving sections..");
	
	var appDb = dirtkey('opa.db'),
		sectionsDb = dirtkey('sections.db'),
		section = req.body["section"],
		content = req.body["content"];
	
	if (typeof section!="undefined") {
		sectionsDb.set(section,{content:content}, function() {
			console.log('Added content', sectionsDb.get(section));
			res.redirect('/admin');
		});
	}
	else {
		res.redirect('/error');
	}

};

exports.admin.snippets = function(req,res){
	console.log("getting snippet..");
	locals.date = new Date().toLocaleDateString();

	var appDb = dirtkey('opa.db'),
		key = req.params["k"];
		
	locals.id = key;
	
	if (typeof key!="undefined") {
		appDb.on('load', function() {
			var sectionsDb = dirtkey('sections.db');
			sectionsDb.on('load', function() {
				var snippetsDb = dirtkey('snippet.db');
				snippetsDb.on('load', function() {
					res.render("admin",{locals:locals,snippets:snippetsDb,sections:sectionsDb,app:appDb.get('app'),page:appDb.get('page'),snippet:snippetsDb.get(key)});
				});
			});
		});
	}
	else {
		res.render('error');
	}
	
};

exports.admin.snippets.post = function(req,res){
	console.log("saving snippet..");

	var appDb = dirtkey('opa.db'),
		snippetsDb = dirtkey('snippet.db'),
		key = req.params["k"],
		vals = req.body;
	
	if (typeof key!="undefined") {
		snippetsDb.on('load', function() {
			snippetsDb.set(key,vals, function() {
				console.log('Snippet saved', snippetsDb.get(key));
			});
			res.send('Thank you.');
		});
	}
	else {
		res.render('error');
	}
};

exports.admin.snippets.del = function(req,res){
	console.log("deleting snippet..");
	
	var appDb = dirtkey('opa.db'),
		snippetsDb = dirtkey('snippet.db'),
		key = req.params["k"];
	
	if (typeof key!="undefined") {
		snippetsDb.rm(key, function() {
			console.log('Deleted snippet');
		});
	}
	
	snippetsDb.on('drain', function() {
		res.redirect('/admin');
	});
}

exports.contact = function(req,res){
    
}

exports.contact.post = function(req,res){
	console.log('contact form submit..');
	
	var appDb = dirtkey('opa.db'),
        postDb = dirtkey('post.db');
	
	var email = req.body["email"],
		name = req.body["name"],
		notes = req.body["notes"],
		timestamp = new Date().getTime();
	
	if (email==='' && name==='' && notes==='') {
		res.send({err:"The form is empty. Please fill it out before sending."});
	}
	else if (email==='' || typeof email==="undefined") {
		res.send({err:"Please let us know your email so that we can contact you."});
	}
	else {
		if (validateEmail(email)){
			appDb.on('load', function() {
				var key = timestamp,
					val = req.body;
					val.username = req.session.username; //append current user if avail
					val.ip = req.connection.remoteAddress;

				if (typeof key!="undefined") {
					postDb.set(key,val, function() {
						//console.log('Post saved', postDb.get(key));
						res.send({msg:'Thanks, we got it.'});
					});
				}
				else {
					res.send({err:"No key error."});
				}
			});	
		}
		else {
			res.send({err:"That email doesn't seem to be valid."});
		}
	}
};

/* utils */
function validateEmail (value) {
  var isValid = /^((([a-z]|\d|[!#\$%&'\*\+\-\/=\?\^_`{\|}~]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])+(\.([a-z]|\d|[!#\$%&'\*\+\-\/=\?\^_`{\|}~]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])+)*)|((\x22)((((\x20|\x09)*(\x0d\x0a))?(\x20|\x09)+)?(([\x01-\x08\x0b\x0c\x0e-\x1f\x7f]|\x21|[\x23-\x5b]|[\x5d-\x7e]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(\\([\x01-\x09\x0b\x0c\x0d-\x7f]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF]))))*(((\x20|\x09)*(\x0d\x0a))?(\x20|\x09)+)?(\x22)))@((([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])*([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])))\.)+(([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])*([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])))\.?$/i.test(value);
  return isValid;
}

==> one-page-app-0.1.2.tgz: package/index.js <==
var dirtkey = require('dirtkey'),
    conf = require('./conf.js'),
    dirtkeyConf = new(dirtkey.conf)({host:conf.database.host,port:conf.database.port,db:conf.database.name});

var locals = {
    author:'in1'
};

function initApp(req,res){
    
    // this is run once to setup new app or to reset all app data
    
    // TODO: one time copy of orig data files
    //fs.createReadStream(__dirname + '/opa.db_').pipe(fs.createWriteStream(__dirname + '/opa.db'));
    //fs.createReadStream(__dirname + '/sections.db_').pipe(fs.createWriteStream(__dirname + '/section.db'));
    //fs.createReadStream(__dirname + '/snippet.db_').pipe(fs.createWriteStream(__dirname + '/snippet.db'));
    //fs.createReadStream(__dirname + '/user.db_').pipe(fs.createWriteStream(__dirname + '/user.db'));
    
    res.redirect("/admin");
}

exports.index = function(req,res){
    console.log("show home page");

    var appDb = dirtkey('opa.db'),
        pageDb = dirtkey('page.db'),
        sectionsDb;
        
    appDb.on('load', function() {
        
        if (typeof appDb._docs === "undefined"){
            initApp(req,res);
        }
        
        sectionsDb = dirtkey('sections.db');
        sectionsDb.on('load', function() {
            if (req.session && typeof req.session.once=="undefined") {
                locals.once=pageDb.once;
                req.session.once=1;
            }
            res.render('index.ejs', {locals:locals,sections:sectionsDb,app:appDb._docs,page:pageDb._docs,msg:req.query.msg,err:req.query.err});
        });
    });
};

exports.login = function(req,res){
    console.log("show login page");

	var appDb = dirtkey('opa.db'),
        pageDb = dirtkey('page.db'),
		sectionsDb;
		
	appDb.on('load', function() {
		sectionsDb = dirtkey('sections.db');
		sectionsDb.on('load', function() {
			res.render('login', {locals:locals,sections:sectionsDb,app:appDb._docs,page:pageDb._docs,err:req.query["err"]});
		});
	});
};

exports.login.post = function(req,res){
	console.log("logging in");
		
	var userDb = dirtkey('user.db'),
		username = req.body["username"],
		password = req.body["password"];
		
	if (typeof username=="undefined" || typeof password=="undefined") {
		res.redirect('/login?err=You need to enter a username and password.');
	}
	else {
		userDb.on('load', function() {
			var user = userDb.get(username);
			if (typeof user=="undefined" || user.password!=password) {
				res.redirect('/login?err=User not found or wrong password.');
			}
			else if (!user.admin && user.admin!=1) {
				res.redirect('/login?err=You do not have permission to access this page.');
			}
			else {
				/* set session login here*/
				req.session.loggedIn=true;
				req.session.username=username;
				res.redirect('/admin');
			}
		});
	}
};

exports.logout = function(req,res){
	console.log("logging out");
    
	delete req.session.loggedIn;
	res.redirect('/');
};

exports.admin = function(req,res){
	console.log("getting admin page..");

    locals.date = new Date().toLocaleDateString();
		
	if (req.session.loggedIn) {
		var appDb = dirtkey('opa.db');
		appDb.on('load', function() {
            var pageDb = dirtkey('page.db'),
                sectionsDb = dirtkey('sections.db');
			
            if (typeof appDb._docs === "undefined"){
				initApp(req,res);
			}
            
            sectionsDb.on('load', function() {
				
				var snippetsDb = dirtkey('snippet.db');
				snippetsDb.on('load', function() {
					res.render('admin', {locals:locals,sections:sectionsDb,snippets:snippetsDb,app:appDb._docs,page:pageDb._docs,msg:req.query.msg,err:req.query.err});
				});
			});
		});
		
	}
	else {
		res.redirect('/login');
	}
};

exports.admin.app = function(req,res){
	console.log("saving app settings..");

	var appDb = dirtkey('opa.db'),
		pageDb = dirtkey('page.db');
		
	appDb.on('load', function() {
		console.log('save app----');
		appDb.set('app',req.body, function() {
			console.log('Added %s.', appDb.get('app'));
		});
	});
	
	appDb.on('drain', function() {
		console.log('app is saved on disk.');
		
		var sectionsDb = dirtkey('sections.db');
		sectionsDb.on('load', function() {
			
			var snippetsDb = dirtkey('snippet.db');
			snippetsDb.on('load', function() {
				res.render('admin', {locals:locals,sections:sectionsDb,snippets:snippetsDb,app:appDb._docs,page:pageDb._docs,msg:"App Settings Saved."});
			});
		});
	
		//res.redirect("/admin");
	});
};

exports.admin.page = function(req,res){
	console.log("saving page settings..");

	var appDb = dirtkey('opa.db');
	console.log('save page.');
	
	appDb.on('load', function() {
		appDb.set('page',req.body, function() {
			console.log('Saved page %s.', appDb.get('app'));
		});
	});
	
	appDb.on('drain', function() {
		console.log('app page is saved on disk.');
		res.redirect("/admin");
	});
};

exports.admin.sections = function(req,res){
	console.log("creating new section..");

	var sectionsDb = dirtkey('sections.db'),
		key = req.body["section"],
		vals = req.body;
		vals.cssClass = "fill"; // always add this to the CSS class
			
	if (typeof key==="undefined" || key==="") {
		res.redirect("/admin#form"+key);
	}
	else {
		key = key.replace(/[^a-zA-Z0-9]/g,''); // strip special chars.
		sectionsDb.on('load', function() {
			sectionsDb.set(key,vals, function() {
				console.log('Added content %s', sectionsDb.get(key));
				res.redirect("/admin#form"+key);
				
			});
		});
	}
};

exports.admin.sections.post = function(req,res){
	console.log("saving section..");

	var	sectionsDb = dirtkey('sections.db'),
		key = req.params["k"],
		vals = req.body;
	
	console.log("saving section:"+key);
	
	if (typeof key!="undefined") {
		sectionsDb.on('load', function() {
			sectionsDb.set(key,vals, function() {
				console.log('Added content', sectionsDb.get(key));
			});
			res.send('Section saved.');
		});
	}
	else {
		res.send('Section not saved.');
	}
	
};

exports.admin.sections.del = function(req,res){
	console.log("deleting section..");

	var sectionsDb = dirtkey('sections.db'),
		key = req.params["k"],
		section = req.body["section"];
	
	if (typeof key!="undefined") {
		sectionsDb.rm(key, function() {
			console.log('Deleted section');
		});
		
		sectionsDb.on('drain', function() {
			res.send('Section deleted.');
		});
	}
	else {
		res.send('Section delete failed.')
	}
};

exports.admin.sections.all = function(req,res){
	console.log("saving all sections..");

	var appDb = dirtkey('opa.db'),
		sectionsDb = dirtkey('sections.db'),
		sections = req.body["section"],
		contents = req.body["content"];
		
	var i=0;
	sections.forEach(function() {
		sectionsDb.set(sections[i],{content:contents[i]}, function() {
			console.log('Added content %s.', sectionsDb.get(sections[i]));
		});
		i++;
	});
	
	res.render('admin', {locals:locals,app:appDb,sections:sectionsDb,msg:"Saved Sections"});
};

exports.admin.sections.all.post = function(req,res){
	console.log("saving sections..");
	
	var appDb = dirtkey('opa.db'),
		sectionsDb = dirtkey('sections.db'),
		section = req.body["section"],
		content = req.body["content"];
	
	if (typeof section!="undefined") {
		sectionsDb.set(section,{content:content}, function() {
			console.log('Added content', sectionsDb.get(section));
			res.redirect('/admin');
		});
	}
	else {
		res.redirect('/error');
	}

};

exports.admin.snippets = function(req,res){
	console.log("getting snippet..");
	locals.date = new Date().toLocaleDateString();

	var appDb = dirtkey('opa.db'),
		key = req.params["k"];
		
	locals.id = key;
	
	if (typeof key!="undefined") {
		appDb.on('load', function() {
			var sectionsDb = dirtkey('sections.db'),
				pageDb = dirtkey('page.db');
				
			sectionsDb.on('load', function() {
				var snippetsDb = dirtkey('snippet.db');
				snippetsDb.on('load', function() {
					res.render("admin",{locals:locals,snippets:snippetsDb,sections:sectionsDb,app:appDb._docs,page:pageDb._docs,snippet:snippetsDb.get(key)});
				});
			});
		});
	}
	else {
		res.render('error');
	}
	
};

exports.admin.snippets.post = function(req,res){
	console.log("saving snippet..");

	var appDb = dirtkey('opa.db'),
		snippetsDb = dirtkey('snippet.db'),
		key = req.params["k"],
		vals = req.body;
	
	if (typeof key!="undefined") {
		snippetsDb.on('load', function() {
			snippetsDb.set(key,vals, function() {
				console.log('Snippet saved', snippetsDb.get(key));
			});
			res.send('Thank you.');
		});
	}
	else {
		res.render('error');
	}
};

exports.admin.snippets.del = function(req,res){
	console.log("deleting snippet..");
	
	var appDb = dirtkey('opa.db'),
		snippetsDb = dirtkey('snippet.db'),
		key = req.params["k"];
	
	if (typeof key!="undefined") {
		snippetsDb.rm(key, function() {
			console.log('Deleted snippet');
		});
	}
	
	snippetsDb.on('drain', function() {
		res.redirect('/admin');
	});
}

exports.contact = function(req,res){
    
}

exports.contact.post = function(req,res){
	console.log('contact form submit..');
	
	var appDb = dirtkey('opa.db'),
        postDb = dirtkey('post.db');
	
	var email = req.body["email"],
		name = req.body["name"],
		notes = req.body["notes"],
		timestamp = new Date().getTime();
	
	if (email==='' && name==='' && notes==='') {
		res.send({err:"The form is empty. Please fill it out before sending."});
	}
	else if (email==='' || typeof email==="undefined") {
		res.send({err:"Please let us know your email so that we can contact you."});
	}
	else {
		if (validateEmail(email)){
			appDb.on('load', function() {
				var key = timestamp,
					val = req.body;
					val.username = req.session.username; //append current user if avail
					val.ip = req.connection.remoteAddress;

				if (typeof key!="undefined") {
					postDb.set(key,val, function() {
						//console.log('Post saved', postDb.get(key));
						res.send({msg:'Thanks, we got it.'});
					});
				}
				else {
					res.send({err:"No key error."});
				}
			});	
		}
		else {
			res.send({err:"That email doesn't seem to be valid."});
		}
	}
};

/* utils */
function validateEmail (value) {
  var isValid = /^((([a-z]|\d|[!#\$%&'\*\+\-\/=\?\^_`{\|}~]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])+(\.([a-z]|\d|[!#\$%&'\*\+\-\/=\?\^_`{\|}~]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])+)*)|((\x22)((((\x20|\x09)*(\x0d\x0a))?(\x20|\x09)+)?(([\x01-\x08\x0b\x0c\x0e-\x1f\x7f]|\x21|[\x23-\x5b]|[\x5d-\x7e]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(\\([\x01-\x09\x0b\x0c\x0d-\x7f]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF]))))*(((\x20|\x09)*(\x0d\x0a))?(\x20|\x09)+)?(\x22)))@((([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])*([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])))\.)+(([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])*([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])))\.?$/i.test(value);
  return isValid;
}
